@@ -1,4 +1,6 @@
 
+import sys
+
 from .error import *
 from .classes import *
 from .debug import *
@@ -9,11 +11,10 @@ from .text import *
 from .log import *
 from .platform import *
 from .setup import *
+from .contrib.version import Version
 
 #----------------------------------------------------------------------------------------------
 
-import sys
-
 class global_injector:
     def __init__(self):
         try:
@@ -38,4 +39,4 @@ Global.ctor = ctor
 Global.noctor = noctor
 Global.foldl = foldl
 Global.foldr = foldr
-Global.ENV = os.environ
+Global.ENV = Env()
