@@ -48,35 +48,47 @@ class Runner:
         self.output = OutputMode(output)
 
     # sudo: True/False/"file"
-    def run(self, cmd, at=None, output=None, nop=None, _try=False, sudo=False):
+    def run(self, cmd, at=None, output=None, nop=None, _try=False, sudo=False, echo=True):
+        # We're running cmd(s) with a login shell ("bash -l") in order to run profile.d
+        # scripts (installation commands may add such scripts and subsequent installation
+        # commands may rely on them).
+        # Howerver, "bash -l" will wreck PATH of active virtualenvs, thus python scripts will
+        # fail. So if we're in one (i.e. VIRTUAL_ENV is not empty) PATH sould be restored
+        # by re-invoking the activation script.
         if (self.is_root or not self.has_sudo) and sudo is not False:
             sudo = False
         if output is None:
             output = self.output
         else:
             output = OutputMode(output)
+        venv = ENV['VIRTUAL_ENV']
         cmd_file = None
         if cmd.find('\n') > -1:
             cmds1 = str.lstrip(textwrap.dedent(cmd))
-            cmds = filter(lambda s: str.lstrip(s) != '', cmds1.split("\n"))
+            cmds = list(filter(lambda s: str.lstrip(s) != '', cmds1.split("\n")))
+            if venv != '':
+                cmds = [". {VENV}/bin/activate".format(VENV=venv)] + cmds
             cmd = "; ".join(cmds)
             cmd_for_log = cmd
             if sudo is not False:
                 cmd_file = paella.tempfilepath()
                 paella.fwrite(cmd_file, cmd)
-                cmd = "bash {}".format(cmd_file)
+                cmd = "bash -l {}".format(cmd_file)
                 cmd_for_log = "sudo { %s }" % cmd_for_log
         else:
+            if venv != '':
+                cmd = "{{ . {VENV}/bin/activate; {CMD}; }}".format(VENV=venv, CMD=cmd)
             cmd_for_log = cmd
         if sudo is not False:
             if sudo == "file":
                 cmd_file = paella.tempfilepath()
                 paella.fwrite(cmd_file, cmd)
-                cmd = "sudo bash {}".format(cmd_file)
+                cmd = "sudo bash -l {}".format(cmd_file)
                 cmd_for_log = "sudo { %s }" % cmd_for_log
             else:
-                cmd = "sudo " + cmd
-        print(cmd)
+                cmd = "sudo bash -l -c '{}'".format(cmd)
+        if echo:
+            print(cmd)
         if cmd_file is not None:
             print("# {}".format(cmd_for_log))
         sys.stdout.flush()
@@ -89,10 +101,11 @@ class Runner:
             os.close(fd)
             cmd = "{{ {CMD}; }} >{LOG} 2>&1".format(CMD=cmd, LOG=temppath)
         if at is None:
-            rc = subprocess.call(["bash", "-e", "-c", cmd])
+            rc = subprocess.call(["bash", "-l", "-e", "-c", cmd])
         else:
             with cwd(at):
-                rc = os.system(cmd)
+                # rc = os.system(cmd)
+                rc = subprocess.call(["bash", "-l", "-e", "-c", cmd])
         if rc > 0:
             if output != True:
                 if output.on_error():
@@ -144,7 +157,7 @@ class PackageManager(object):
         else:
             raise Error("Cannot determine package manager for OS %s" % platform.os)
 
-    def run(self, cmd, at=None, output="on_error", nop=None, _try=False, sudo=False):
+    def run(self, cmd, at=None, output="on_error", nop=None, _try=False, sudo=False, echo=True):
         return self.runner.run(cmd, at=at, output=output, nop=nop, _try=_try, sudo=sudo)
 
     def has_command(self, cmd):
@@ -278,17 +291,19 @@ class Brew(PackageManager):
     def __init__(self, runner):
         super(Brew, self).__init__(runner)
 
+        # prevents brew from performing auto updates
+        os.environ["HOMEBREW_NO_AUTO_UPDATE"] = "1"
+
         if os.getuid() == 0 and os.getenv("BREW_AS_ROOT") != "1":
             eprint("Cannot run as root. Set BREW_AS_ROOT=1 to override.")
             sys.exit(1)
         if sh('xcode-select -p') == '':
             eprint("Xcode tools are not installed. Please run xcode-select --install.")
             sys.exit(1)
-        if 'VIRTUAL_ENV' not in os.environ:
-            # required because osx pip installed are done with --user
-            os.environ["PATH"] = os.environ["PATH"] + ':' + os.environ["HOME"] + '/Library/Python/2.7/bin'
-        # this prevents brew updating before each install
-        os.environ["HOMEBREW_NO_AUTO_UPDATE"] = "1"
+        if sys.version_info < (3, 0):
+            if 'VIRTUAL_ENV' not in os.environ:
+                # required because osx pip installed are done with --user
+                os.environ["PATH"] = os.environ["PATH"] + ':' + os.environ["HOME"] + '/Library/Python/2.7/bin'
 
     def install(self, packs, group=False, output="on_error", _try=False):
         # brew will fail if package is already installed
@@ -343,7 +358,7 @@ class Alpine(PackageManager):
 #----------------------------------------------------------------------------------------------
 
 class Setup(OnPlatform):
-    def __init__(self, nop=False, verbose=False):
+    def __init__(self, nop=False, verbose=False, sudo=True):
         OnPlatform.__init__(self)
         self.verbose = verbose
         self.nop = nop
@@ -367,6 +382,8 @@ class Setup(OnPlatform):
         self.python = sys.executable
         os.environ["PYTHONWARNINGS"] = 'ignore:DEPRECATION::pip._internal.cli.base_command'
 
+        self.sudoIf(sudo)
+
     def setup(self):
         if self.repo_refresh:
             self.package_manager.update()
@@ -380,8 +397,8 @@ class Setup(OnPlatform):
 
         self.invoke()
 
-    def run(self, cmd, at=None, output="on_error", nop=None, _try=False, sudo=False):
-        return self.runner.run(cmd, at=at, output=output, nop=nop, _try=_try, sudo=sudo)
+    def run(self, cmd, at=None, output="on_error", nop=None, _try=False, sudo=False, echo=True):
+        return self.runner.run(cmd, at=at, output=output, nop=nop, _try=_try, sudo=sudo, echo=echo)
 
     @staticmethod
     def has_command(cmd):
@@ -400,9 +417,10 @@ class Setup(OnPlatform):
         d = self.profile_d
         if as_file is None:
             as_file = os.path.basename(file)
+        not_mac = self.os != 'macos'
         if not os.path.isdir(d):
-            self.run('mkdir -p "{}"'.format(d), sudo=True)
-        self.run('cp "{FROM}" "{TO}"'.format(FROM=file, TO=os.path.join(d, as_file)), sudo=True)
+            self.run('mkdir -p "{}"'.format(d), sudo=not_mac)
+        self.run('cp "{FROM}" "{TO}"'.format(FROM=file, TO=os.path.join(d, as_file)), sudo=not_mac)
 
     def cat_to_profile_d(self, text, as_file=None):
         file = paella.tempfilepath()
@@ -410,11 +428,16 @@ class Setup(OnPlatform):
         d = self.profile_d
         if as_file is None:
             as_file = os.path.basename(file)
+        not_mac = self.os != 'macos'
         if not os.path.isdir(d):
-            self.run('mkdir -p "{}"'.format(d), sudo=True)
-        self.run('cp "{FROM}" "{TO}"'.format(FROM=file, TO=os.path.join(d, as_file)), sudo=True)
+            self.run('mkdir -p "{}"'.format(d), sudo=not_mac)
+        self.run('cp "{FROM}" "{TO}"'.format(FROM=file, TO=os.path.join(d, as_file)), sudo=not_mac)
         os.unlink(file)
 
+    def sudoIf(self, sudo=True):
+        if sudo:
+            self.run("true", sudo=True, echo=False)
+
     #------------------------------------------------------------------------------------------
 
     def install(self, packs, group=False, output="on_error", _try=False, **kwargs):
@@ -480,14 +503,14 @@ class Setup(OnPlatform):
         path = "/usr/local/bin"
         if self.os == 'macos':
             packs= "make coreutils findutils gnu-sed gnu-tar gawk gpatch"
-            path = os.path.abspath(os.path.join(os.path.expanduser("~"), ".cache", "readies", "bin"))
+            path = os.path.abspath(os.path.join(os.path.expanduser("~"), ".local", "bin"))
             if not os.path.isdir(path):
                 os.makedirs(path)
         elif self.os == 'freebsd':
             packs = "gmake coreutils findutils gsed gtar gawk"
         self.install(packs)
 
-        for x in ['make', 'find', 'sed', 'tar', 'mktemp', 'du']:
+        for x in ['make', 'find', 'xargs', 'sed', 'tar', 'mktemp', 'du']:
             dest = os.path.join(path, x)
             if not os.path.exists(dest):
                 src = paella.sh("command -v g{}".format(x)).strip()
