@@ -1,6 +1,8 @@
 
+import os
 import sys
 from subprocess import Popen, PIPE
+import tempfile
 
 def eprint(*args, **kwargs):
     print(*args, file=sys.stderr, **kwargs)
@@ -13,8 +15,18 @@ class ShError(Exception):
 
 def sh(cmd, join=False, lines=False, fail=True):
     shell = isinstance(cmd, str)
+    if shell:
+        # Popen with shell=True defaults to /bin/sh so in order to use bash and
+        # avoid quoting problems we write cmd into a temp file
+        fd, cmd_file = tempfile.mkstemp(prefix='/tmp/sh')
+        with open(cmd_file, 'w') as file:
+            file.write(cmd)
+        os.close(fd)
+        cmd = "/usr/bin/env bash {cmd_file}".format(cmd_file=cmd_file)
     proc = Popen(cmd, shell=shell, stdout=PIPE, stderr=PIPE)
     out, err = proc.communicate()
+    if shell:
+        os.unlink(cmd_file)
     out = out.decode('utf-8').strip()
     if lines is True:
         join = False
